#ifndef __XBEEPACKETS_H__
#define __XBEEPACKETS_H__
#include "serialport.h"
#include <iostream>
#include <conio.h>
#include <windows.h>
using namespace std;

#define API_MODE_2

#define NUMBER_OF_AGENTS 30

#define NUM_DATA_SETS 5
#define NUMBERS_PER_SET 2
#define DATATYPE_BYTELENGTH 4
#define ADDITIONAL_NUMS 5
#define DATA_ARRAY_LENGTH (NUM_DATA_SETS*NUMBERS_PER_SET+ADDITIONAL_NUMS)


#define LOAD_TEMP_BYTE 		if ( !getByte(tempByte) ){return -1;}

typedef struct XBeeInPacket{
	unsigned int length;
	unsigned char API_ID;
	unsigned int source;
	unsigned char RSSI;
	unsigned char options;
	union packetFloat{
		float dataFloat;
		unsigned char dataChar[4];
	} data[DATA_ARRAY_LENGTH];
	unsigned char checksum;
}XBeeInPacket;

class PacketHandler{
public:
	//Constructor takes serial port to bind to as argument
	PacketHandler (CSerialPort& port){
		pCOM = &port;
		numTotalPackets = 0;
		numDroppedPackets=0;
		numCorruptPackets=0;
		for (int i=0;i<NUMBER_OF_AGENTS;i++){
			receivedStats[i]=0;
		}
	}
	int getNumPackets(void){
		return numTotalPackets;
	}
	int getNumDropped(void){
		return numDroppedPackets;
	}
	int getNumCorrupt(void){
		return numCorruptPackets;
	}
	int getReceivedStats(int i){
		return receivedStats[i];
	}
	void resetStats(void){
		numTotalPackets = 0;
		numDroppedPackets=0;
		numCorruptPackets=0;
		for (int i=0;i<NUMBER_OF_AGENTS;i++){
			receivedStats[i]=0;
		}
	}
	//get a packet
	int getPacket( XBeeInPacket& inPacket ){

		//return value Error Codes//
		//0: successfully read
		//-1: action canceled by user
		//-2: start delimiter wrong, aborted by user
		//-3: checksum incorrect
		//-4: source number invalid
		//-5: read fail

		unsigned char tempByte;
		unsigned char verifySum=0;
		//cout<<"getting packet, press 'a' to abort..."<<endl;
		//cout<<"press 'r' to reset stats...<<endl;
		pCOM->GetStatus(comstat);
		while (comstat.cbInQue <= 100){
			pCOM->GetStatus(comstat);
			if (_kbhit()){
				if (_getch()=='a'){
					return -1;
				}
			}
		}

		numTotalPackets++;

		pCOM->Read(&tempByte,1);
		if ( tempByte != 0x7E ){
			cout<<"start delimiter wrong, packet dropped."<<endl;
			cout<<"Waiting for next packet, press 'a' to abort"<<endl;
			numDroppedPackets++;
			while ( tempByte!=0x7E ){
				if (_kbhit()){
					if (_getch()=='a'){
						return -1;
					}
				}

				else{
					pCOM->Read(&tempByte,1);//read to find next 0x7E
				}
				
			}

		}
		LOAD_TEMP_BYTE
			inPacket.length=tempByte*256;
		LOAD_TEMP_BYTE
			inPacket.length=tempByte+inPacket.length;
		LOAD_TEMP_BYTE
			inPacket.API_ID=tempByte;
		LOAD_TEMP_BYTE
			inPacket.source=tempByte*256;
		LOAD_TEMP_BYTE
			inPacket.source=inPacket.source+tempByte;
		LOAD_TEMP_BYTE
			inPacket.RSSI=tempByte;
		LOAD_TEMP_BYTE
			inPacket.options=tempByte;
		//load bytes into the data union.  4 "chars" make up one "float"
		for (int i=0; i<DATA_ARRAY_LENGTH; i++){	
			for (int j=0; j<=3; j++){
				LOAD_TEMP_BYTE
				inPacket.data[i].dataChar[3-j]=tempByte;
				verifySum+=inPacket.data[i].dataChar[3-j];
			}
		}
		LOAD_TEMP_BYTE	
		inPacket.checksum=tempByte;

		verifySum+=inPacket.API_ID 
				+ inPacket.source 
				+ inPacket.RSSI 
				+ inPacket.options 
				+ inPacket.checksum;

		if (verifySum == 0xFF){
			if (inPacket.source < NUMBER_OF_AGENTS){
				cout<<"Checksum correct!"<<endl;
				receivedStats[inPacket.source]++;
				return 0;
			}
			else{
				cout<<"Source wrong!"<<endl;
				numCorruptPackets++;
				return -4;
			}

		}
		else{
			cout<<"Checksum incorrect!"<<endl;
			numCorruptPackets++;
			return -3;	
		}
	}

	CSerialPort* pCOM;
private:
	//variables

	COMSTAT comstat;
	int receivedStats[NUMBER_OF_AGENTS];
	int numDroppedPackets; //Encountered 0x7E before the end of packet
	int numCorruptPackets; //Checksum is wrong
	int numTotalPackets;
	COMSTAT getComstat(void){
		return comstat;
	}

	//functions+
	int getByte( unsigned char & charBuffer ){
		//cout<<"getting byte..."<<endl;
		bool messageFlag=0;
		pCOM->GetStatus(comstat);
		while( comstat.cbInQue==0 ){ 
			if (_kbhit()){
				char tempchar = _getch();
				if (tempchar=='a'){
					return 0;
				}
				else if (tempchar=='r'){
					resetStats();
				}
			}
			pCOM->GetStatus(comstat);
		}
		pCOM->Read( &charBuffer,1 );
#if defined API_MODE_2
		if (charBuffer==0x7D){
			while( comstat.cbInQue==0 ){ 
				if (_kbhit()){
					char tempchar = _getch();
					if (tempchar=='a'){
						return 0;
					}
					else if (tempchar=='r'){
						resetStats();
					}
				}
				pCOM->GetStatus(comstat);
			}
			pCOM->Read( &charBuffer,1 );
			charBuffer=charBuffer^0x20;
		}
#endif
		return 1;
	}
};

#endif